var LabEnvAlarm = function()
{
	var labenvalarm = {};
	var debug = 1;
	var sensors;

	// Page initialization
	labenvalarm.initPage = function()
	{
		// Navigation
		Buttons.addClickHandler('gosetalarmblocker', Wizard.goRegister);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);

		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', labenvalarm.submit);

		// Create lab environment alarms web page from configuration data
		labenvalarm.getAlarmInfo();
	}

	labenvalarm.getAlarmInfo = function()
	{
		// Get lab sensor configurations JSON object with Ajax
		var url = '../LabEnvironment.servlet?ID='+App.getSessionId();
		url += '&cmd=GetLabSensorConfigList';
		Wizard.showLoadingAnimation('Loading sensor information. Please wait...');
		Wizard.asyncJsonRequest(url, labenvalarm.onAlarmInfoLoaded);
	}

	labenvalarm.onAlarmInfoLoaded = function(response)
	{
		var template = Doc.element('row-template');
		var cols = template.getElementsByTagName('th');
		var rows = Doc.element('all-rows');

		// Get sensor configurations information from the Ajax response
		sensors = response.sensors;
		var hasAlarmPermission = false;
		for (var sensorNo = 0; sensorNo < sensors.length; sensorNo++)
		{
			var sensor = sensors[sensorNo];
			
			var alarms = sensor.alarms;
			for (var alarmNo = 0; alarmNo < alarms.length; alarmNo++)
			{
				var alarm = alarms[alarmNo];
				
				var row = document.createElement('tr');
				row.className = sensorNo % 2 == 0 ? 'evenrow highlight' : 'oddrow highlight';

				for (var colNo = 0; colNo < cols.length; colNo++)
				{
					var cell = document.createElement('td');
					cell.className = 'cell';
					var colId = cols[colNo].id;
					if (colId == 'alarms.alarmId')
					{
						cell.className = 'id-col';
						cell.innerHTML = alarm.no;
					}
					else if (colId == 'alarms.sensorName')
					{
						cell.innerHTML = Strings.encodeTags(sensor.name);
					}
					else if (colId == 'alarms.url')
					{
						cell.innerHTML = '<a href="'+Strings.encodeTags(sensor.url)+'" target="_blank" rel="noopener noreferrer">'+Strings.encodeTags(sensor.url)+'</a>';
					}
					else if (colId == 'alarms.startTime')
					{
						cell.innerHTML = Strings.encodeTags(alarm.startTime);
					}
					else if (colId == 'alarms.endTime')
					{
						cell.innerHTML = Strings.encodeTags(alarm.endTime);
					}
					else if (colId == 'alarms.weekdayFilter')
					{
						cell.innerHTML = Strings.encodeTags(alarm.weekdayFilter);
					}
					else if (colId == 'alarms.minTemp')
					{
						cell.innerHTML = Numbers.formatNumber(alarm.temperatureMin, 1);
					}
					else if (colId == 'alarms.maxTemp')
					{
						cell.innerHTML = Numbers.formatNumber(alarm.temperatureMax, 1);
					}
					else if (colId == 'alarms.minHumidity')
					{
						cell.innerHTML = Numbers.formatNumber(alarm.humidityMin, 1);
					}
					else if (colId == 'alarms.maxHumidity')
					{
						cell.innerHTML = Numbers.formatNumber(alarm.humidityMax, 1);
					}
					else if (colId == 'alarms.blocked')
					{
						if (alarm.alarmPermission)
						{
							hasAlarmPermission = true;
							var html = '<select name="alarmBlocked.'+alarm.no+'">';
							html += '<option value="1"' + (alarm.blocked ? 'selected' :'') + '>Yes';
							html += '<option value="0"' + (alarm.blocked ? '' : 'selected') + '>No';
							html += '</select>';
							cell.innerHTML = html;
						}
						else
						{
							cell.innerHTML = alarm.blocked ? 'Yes' : 'No';
						}
					}
					else if (colId == 'alarms.alarmText')
					{
						cell.innerHTML = Strings.encodeTags(alarm.alarmText);
					}
					row.appendChild(cell);
				}
				rows.appendChild(row);
				
			}
		}

		Doc.show('alarms');
		if (hasAlarmPermission)
		{
			Doc.show("gosetalarmblocker");
		}
	}

	labenvalarm.submit = function()
	{
		var submitInfo = {};
		var frm = document.forms['alarms'];
		
		// Get sensor configurations information from the Ajax response
		for (var sensorNo = 0; sensorNo < sensors.length; sensorNo++)
		{
			var sensor = sensors[sensorNo];
			
			var alarms = sensor.alarms;
			for (var alarmNo = 0; alarmNo < alarms.length; alarmNo++)
			{
				var alarm = alarms[alarmNo];
				frm['alarmBlocked.'+alarm.no].value;
				var tmp = {};
				tmp.blocked = parseInt(frm['alarmBlocked.'+alarm.no].value) ? true : false;
				submitInfo['alarm.'+alarm.no] = tmp;
			}
		}

		var request = Ajax.getXmlHttpRequest();
		var url = '../LabEnvironment.servlet?ID='+App.getSessionId();
		url += '&cmd=SetLabEnvAlarmConfig';
		Wizard.showLoadingAnimation('Updating alarm status. Please wait...');
		Wizard.asyncJsonRequest(url, labenvalarm.submissionResults, 'POST', JSON.stringify(submitInfo));
	}

	labenvalarm.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return labenvalarm;
}();

Doc.onLoad(LabEnvAlarm.initPage);